// TODO: 干掉 api.cesium.com
// XXX: ion-sdk ？？？

import * as Cesium from 'cesium';
import 'cesium/Build/Cesium/Widgets/widgets.css';

Object.assign(window, { Cesium });

_configureCesium();

export function cesium_init(container: Element) {
  const viewer = new Cesium.Viewer(container, {
    animation: true, // .cesium-viewer-animationContainer https://cesium.com/learn/ion-sdk/ref-doc/Animation.html
    ...{
      baseLayer: Cesium.ImageryLayer.fromProviderAsync(
        Cesium.TileMapServiceImageryProvider.fromUrl(Cesium.buildModuleUrl('Assets/Textures/NaturalEarthII')),
      ),
      baseLayerPicker: !true,
      geocoder: !true, // = IonGeocodeProviderType.DEFAULT] - 在使用Geocoder小部件进行搜索时使用的地理编码服务或服务。如果设置为false，则不会创建Geocoder小部件。
    },
    fullscreenButton: !true, // 全屏按钮
    // globe: false, // 地球
    homeButton: true, // Home按钮
    infoBox: false, // InfoBox小部件。
    navigationHelpButton: false, // 是否显示导航帮助按钮
    orderIndependentTranslucency: false, // 顺序无关透明度
    projectionPicker: !true, // 投影选择器
    requestRenderMode: !true, // 如果为真，渲染帧将仅在场景内部发生变化时需要时发生。启用此功能可以减少应用程序的CPU/GPU使用率，并在移动设备上节省更多电量，但在此模式下需要使用{@link Scene#requestRender}显式渲染新帧。在API的其他部分对场景进行更改后，在许多情况下都需要这样做。请参阅{@link https://cesium.com/blog/2018/01/24/cesium-scene-rendering-performance/|使用显式渲染提高性能}。
    sceneModePicker: true, // 是否显示场景模式选择器(2D/3D切换)
    selectionIndicator: true,
    shadows: true, // Determines if shadows are cast by light sources.
    /* animationContainer: !true, */
    /* timelineContainer: true, */
    /* bottomContainer: document.createElement('p'), // The DOM element or ID that will contain the bottomContainer.  If not specified, the bottomContainer is added to the widget itself. */
    shouldAnimate: !true,
    showRenderLoopErrors: true, // 如果为真，当发生渲染循环错误时，此小部件将自动向用户显示包含错误的HTML面板。
    timeline: true, // If set to false, the Timeline widget will not be created.
  });

  viewer.scene.debugShowFramesPerSecond = true;

  // _configureMapTile(viewer);

  return viewer;
}

function _configureCesium() {
  /* 时间日期格式化 */ {
    const minutes = 0 - new Date().getTimezoneOffset(); // 0 - (-480);

    // Animation 的时间日期格式化
    Cesium.AnimationViewModel.defaultDateFormatter = function (date) {
      const dataZone8 = Cesium.JulianDate.addMinutes(date, minutes, new Cesium.JulianDate());
      return Cesium.JulianDate.toIso8601(dataZone8).slice(0, 10);
    };
    Cesium.AnimationViewModel.defaultTimeFormatter = function (time) {
      const dataZone8 = Cesium.JulianDate.addMinutes(time, minutes, new Cesium.JulianDate());
      return Cesium.JulianDate.toIso8601(dataZone8).slice(11, 19);
    };

    // Timeline 的时间日期格式化
    // @ts-expect-error node_modules/@cesium/widgets/Source/Timeline/Timeline.js
    Cesium.Timeline.prototype.makeLabel = function (time) {
      const dataZone8 = Cesium.JulianDate.addMinutes(time, minutes, new Cesium.JulianDate());
      return Cesium.JulianDate.toIso8601(dataZone8).slice(0, 19);
    };
  }

  // 默认视图区域
  Cesium.Camera.DEFAULT_VIEW_RECTANGLE = Cesium.Rectangle.fromDegrees(
    75, // 西经
    10, // 南纬
    140, // 东经
    60, // 北纬
  );
}

function _configureMapTile(viewer: Cesium.Viewer) {
  const provider = new Cesium.UrlTemplateImageryProvider({
    maximumLevel: 18,
    minimumLevel: 3,
    url: 'https://webst02.is.autonavi.com/appmaptile?style=6&x={x}&y={y}&z={z}',
  });

  if (viewer.baseLayerPicker) {
    // 如果有底图选择器
    const customLayerViewModel = new Cesium.ProviderViewModel({
      category: 'Cesium ion', // 或 'Other 、Cesium ion'、'Bing Maps' 等
      creationFunction() {
        return provider;
      },
      iconUrl: 'gaodeImage.png',
      name: '高德地图',
      tooltip: '高德地图',
    });
    // 设置高德地图为默认图层
    viewer.baseLayerPicker.viewModel.imageryProviderViewModels.unshift(customLayerViewModel);
    const selectedViewModel = viewer.baseLayerPicker.viewModel.imageryProviderViewModels[0];
    if (!selectedViewModel) {
      console.error('未找到默认底图');
      return;
    }
    viewer.baseLayerPicker.viewModel.selectedImagery = selectedViewModel;
  } else {
    // 如果没有底图选择器
    viewer.imageryLayers.removeAll();
    viewer.imageryLayers.addImageryProvider(provider);
  }
}

function _flyToDemo(viewer: Cesium.Viewer) {
  // 将三维球定位到中国
  viewer.camera.flyTo({
    complete: function complete() {},
    destination: Cesium.Cartesian3.fromDegrees(103.84, 31.15, 17_850_000),
    orientation: {
      heading: Cesium.Math.toRadians(348.420_294_285_197_8),
      pitch: Cesium.Math.toRadians(-89.740_266_879_720_41),
      roll: Cesium.Math.toRadians(0),
    },
  });
}
